//---------------------------------------------------------------------------
#ifndef App_diskimagesH
#define App_diskimagesH
//---------------------------------------------------------------------------
#include "App_Common.h"
//---------------------------------------------------------------------------
namespace App_diskimages {
//---------------------------------------------------------------------------
class EDiskImageError : public Exception
{
public:
	__fastcall EDiskImageError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class EInvalidDiskImage : public EDiskImageError
{
public:
	__fastcall EInvalidDiskImage(const AnsiString& AMessage) : EDiskImageError(AMessage) {}
};
//---------------------------------------------------------------------------
class TDiskImageFormat
{
public:
	static const int HeaderOffset = 0x0000;
	static const int HeaderSize = 0x2B0;
	static const int NameOffset = 0x0000;
	static const int NameSize = 17;
	static const int ReserveOffset = 0x0011;
	static const int ReserveSize = 9;
	static const int WriteProtectedOffset = 0x001A;
	static const int WriteProtectedSize = 1;
	static const int DiskTypeOffset = 0x001B;
	static const int DiskTypeSize = 1;
	static const int SizeOffset = 0x001C;
	static const int SizeSize = 4;
	static const int TrackEntrySize = 4;
	static const int TrackEntryCount = 164;
	static const int TrackTableOffset = 0x0020;
	static const int TrackTableSize = TrackEntrySize * TrackEntryCount;
	static const int MaxNameLength = NameSize - 1;
	static const int WriteProtected_No = 0x00;
	static const int WriteProtected_Yes = 0x10;
	static const int DiskType_2D = 0x00;
	static const int DiskType_2DD = 0x10;
	static const int DiskType_2HD = 0x20;
	static int __fastcall ReadInteger(void* AData, int AOffset, int ASize);
	static void __fastcall WriteInteger(void* AData, int AOffset, int ASize, int AValue);
	static AnsiString __fastcall ReadString(void* AData, int AOffset, int ASize);
	static void __fastcall WriteString(void* AData, int AOffset, int ASize, const AnsiString& AValue);
	static const AnsiString& __fastcall AssureValidName(const AnsiString& AName);
	static bool __fastcall IsValidName(const AnsiString& AName);
	static int __fastcall ToggleWriteProtected(int AWriteProtected);
	static int __fastcall AssureValidWriteProtected(int AWriteProtected);
	static bool __fastcall IsValidWriteProtected(int AWriteProtected);
	static int __fastcall AssureValidDiskType(int ADiskType);
	static bool __fastcall IsValidDiskType(int ADiskType);
	static int __fastcall AssureValidSize(int ASize);
	static bool __fastcall IsValidSize(int ASize);
};
//---------------------------------------------------------------------------
class TDiskImage : public TObject
{
private:
	TMemoryStream* FData;
	AnsiString FName;
	int FDiskType;
	int FWriteProtected;
	int FSize;
	void __fastcall Initialize(int ADiskType);
	void __fastcall Uninitialize();
	void __fastcall Create(int ADiskType);
	void __fastcall Update();
	void __fastcall UpdateData();
	void __fastcall SetName(const AnsiString& AName);
	void __fastcall SetDiskType(int ADiskType);
	void __fastcall SetWriteProtected(int AWriteProtected);
	void __fastcall SetSize(int ASize);
	__property TMemoryStream* Data = {read=FData};
protected:
public:
	__fastcall TDiskImage(int ADiskType = TDiskImageFormat::DiskType_2D);
	virtual __fastcall ~TDiskImage();
	virtual void __fastcall Assign(TDiskImage* ASource);
	virtual void __fastcall LoadFromFile(const AnsiString& AFile);
	virtual void __fastcall SaveToFile(const AnsiString& AFile);
	virtual void __fastcall LoadFromStream(TStream* AStream);
	virtual void __fastcall SaveToStream(TStream* AStream);
	__property AnsiString Name = {read=FName, write=SetName};
	__property int DiskType = {read=FDiskType, write=SetDiskType};
	__property int WriteProtected = {read=FWriteProtected, write=SetWriteProtected};
	__property int Size = {read=FSize};
};
//---------------------------------------------------------------------------
class TDiskImageDescriber
{
private:
	static int __fastcall EvaluateWriteProtected(int AWriteProtected);
	static int __fastcall EvaluateDiskType(int ADiskType);
protected:
public:
	static int ImageIndex_WriteProtected_Yes;
	static int ImageIndex_WriteProtected_No;
	static AnsiString __fastcall DescribeByName(TDiskImage* ADiskImage);
	static AnsiString __fastcall DescribeByWriteProtected(TDiskImage* ADiskImage);
	static AnsiString __fastcall DescribeByDiskType(TDiskImage* ADiskImage);
	static AnsiString __fastcall DescribeBySize(TDiskImage* ADiskImage);
	static int __fastcall CompareByName(TDiskImage* ADiskImage1, TDiskImage* ADiskImage2);
	static int __fastcall CompareByWriteProtected(TDiskImage* ADiskImage1, TDiskImage* ADiskImage2);
	static int __fastcall CompareByDiskType(TDiskImage* ADiskImage1, TDiskImage* ADiskImage2);
	static int __fastcall CompareBySize(TDiskImage* ADiskImage1, TDiskImage* ADiskImage2);
	static int __fastcall DetermineImageIndex(TDiskImage* ADiskImage);
};
//---------------------------------------------------------------------------
} // End namespace App_diskimages
using namespace App_diskimages;
//---------------------------------------------------------------------------
#endif

