//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include <ComCtrls.hpp>
#include <Dialogs.hpp>
#include <Forms.hpp>
#include <IniFiles.hpp>
#include "App_Commands.h"
#include "App_Documents.h"
#include "App_History.h"
#include "App_InputDialog.h"
#include "App_SplitDialog.h"
#include "Moa_AutoPointers.h"
#include "Moa_Clipboard.h"
#include "Moa_CommandPattern.h"
#include "Moa_DynamicArrays.h"
#include "Moa_Files.h"
#include "Moa_ListViews.h"
#include "Moa_ShellLinks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_documents {
//---------------------------------------------------------------------------
void __fastcall TDocumentFileModel::SetFile(const AnsiString& AFile)
{
	if(AFile == File) {
		return;
	}
	FFile = AFile;
	SetCurrentDir(ExtractFileDir(File));
	DoChange();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentFileModel::GetTitle()
{
	if(IsOpened()) {
		return ExtractFileName(File);
	}
	else {
		return "Untitled";
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentFileModel::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocumentFileModel::TDocumentFileModel() :
	TModel(),
	FFile(),
	FOnChange(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentFileModel::~TDocumentFileModel()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentFileModel::IsOpened()
{
	return File.Length() > 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TDocumentFileView::TDocumentFileView(TDocumentFileModel* ADocumentFileModel, TForm* AForm) :
	TView(),
	FDocumentFileModel(ADocumentFileModel),
	FForm(AForm)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentFileView::~TDocumentFileView()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDocumentFileView::HandleChange(TObject* ASender)
{
	Form->Caption = AnsiString().sprintf(
		"%s - %s",
		DocumentFileModel->Title.c_str(),
		TCommon::Title.c_str());
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TDocumentListModel::SortCompareByName(int AIndex1, int AIndex2, void* AData)
{
	return TDiskImageDescriber::CompareByName(
		static_cast<TDocumentListModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentListModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentListModel::SortCompareByDiskType(int AIndex1, int AIndex2, void* AData)
{
	return TDiskImageDescriber::CompareByDiskType(
		static_cast<TDocumentListModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentListModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentListModel::SortCompareByWriteProtected(int AIndex1, int AIndex2, void* AData)
{
	return TDiskImageDescriber::CompareByWriteProtected(
		static_cast<TDocumentListModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentListModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentListModel::SortCompareBySize(int AIndex1, int AIndex2, void* AData)
{
	return TDiskImageDescriber::CompareBySize(
		static_cast<TDocumentListModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentListModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::Initialize()
{
	try {
		FList = new TList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TDiskImage* __fastcall TDocumentListModel::GetItem(int AIndex)
{
	return static_cast<TDiskImage*>(List->Items[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentListModel::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::DoInsertItems(const TIntegerDynArray& AIndexes)
{
	if(OnInsertItems != NULL) {
		OnInsertItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::DoDeleteItems(const TIntegerDynArray& AIndexes)
{
	if(OnDeleteItems != NULL) {
		OnDeleteItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::DoClear()
{
	if(OnClear != NULL) {
		OnClear(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::DoReorder(const TListReorderRecords& ARecords)
{
	if(OnReorder != NULL) {
		OnReorder(this, ARecords);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::DoItemChange(int AIndex)
{
	if(OnItemChange != NULL) {
		OnItemChange(this, AIndex);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocumentListModel::TDocumentListModel() :
	TModel(),
	FList(NULL),
	FOnInsertItems(NULL),
	FOnDeleteItems(NULL),
	FOnClear(NULL),
	FOnReorder(NULL),
	FOnItemChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocumentListModel::~TDocumentListModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::AddItems(const DynamicArray<TDiskImage*>& AItems)
{
	if(AItems.Length == 0) {
		return;
	}
	if(List->Capacity < List->Count + AItems.Length) {
		List->Capacity = List->Count + AItems.Length;
	}
	TIntegerDynArray LIndexes;
	LIndexes.Length = AItems.Length;
	for(int LI = 0; LI < AItems.Length; ++LI) {
		LIndexes[LI] = List->Add(AItems[LI]);
	}
	DoInsertItems(LIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::DeleteItems(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		List->Delete(AIndexes[AIndexes.High - LI]);
	}
	DoDeleteItems(AIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::Clear()
{
	if(Count == 0) {
		return;
	}
	List->Clear();
	DoClear();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::SortByName()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByName, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::SortByDiskType()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByDiskType, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::SortByWriteProtected()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByWriteProtected, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::SortBySize()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareBySize, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::ReverseOrder()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::ReverseOrder(Count));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::MoveUp(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	Reorder(TListAlgorithms::Slide(Count, AIndexes, -1));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::MoveDown(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	Reorder(TListAlgorithms::Slide(Count, AIndexes, 1));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::Reorder(const TListReorderRecords& ARecords)
{
	if(ARecords.Length == 0) {
		return;
	}
	TListUtilities::Reorder(List, ARecords);
	DoReorder(ARecords);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListModel::ItemChange(int AIndex)
{
	DoItemChange(AIndex);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentListModel::Find(TDiskImage* ADiskImage)
{
	return List->IndexOf(ADiskImage);
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TDocumentListModel::FindItems(const DynamicArray<TDiskImage*>& AItems)
{
	TIntegerDynArray LIndexes;
	LIndexes.Length = AItems.Length;
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		LIndexes[LI] = Find(AItems[LI]);
	}
	return LIndexes;
}
//---------------------------------------------------------------------------
DynamicArray<TDiskImage*> __fastcall TDocumentListModel::GetItemsAt(const TIntegerDynArray& AIndexes)
{
	DynamicArray<TDiskImage*> LItems;
	LItems.Length = AIndexes.Length;
	for(int LI = 0; LI < LItems.Length; ++LI) {
		LItems[LI] = Items[AIndexes[LI]];
	}
	return LItems;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TDocumentListView::TDocumentListView(TDocumentListModel* ADocumentListModel, TListView* AListView) :
	TView(),
	FDocumentListModel(ADocumentListModel),
	FListView(AListView)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentListView::~TDocumentListView()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListView::HandleInsertItems(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::InsertItems(ListView, AIndexes);
		TListViewItemsUtilities::SetSelectionIndexes(ListView, AIndexes);
		TListViewItemUtilities::SetFocused(ListView, AIndexes[AIndexes.High], true);
		TListViewItemUtilities::EnsureVisible(ListView, AIndexes[AIndexes.High], false);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListView::HandleDeleteItems(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::DeleteItems(ListView, AIndexes);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListView::HandleClear(TObject* ASender)
{
	if(ListView->Items->Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->Items->Count = 0;
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListView::HandleReorder(TObject* ASender, const TListReorderRecords& ARecords)
{
	if(ARecords.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::Reorder(ListView, ARecords);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentListView::HandleItemChange(TObject* ASender, int AIndex)
{
	ListView->UpdateItems(AIndex, AIndex);
	ListView->Update();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TDocumentSelection::GetItem(int AIndex)
{
	return TListViewItemUtilities::IsSelected(ListView, AIndex);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelection::SetItem(int AIndex, bool AItem)
{
	TListViewItemUtilities::SetSelected(ListView, AIndex, AItem);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentSelection::GetCount()
{
	return ListView->Items->Count;
}
//---------------------------------------------------------------------------
int __fastcall TDocumentSelection::GetSelectionCount()
{
	return ListView->SelCount;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TDocumentSelection::GetSelectionIndexes()
{
	return TListViewItemsUtilities::GetSelectionIndexes(ListView);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelection::SetSelectionIndexes(const TIntegerDynArray& ASelectionIndexes)
{
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::SetSelectionIndexes(ListView, ASelectionIndexes);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
__fastcall TDocumentSelection::TDocumentSelection(TListView* AListView) :
	TModel(),
	FListView(AListView)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentSelection::~TDocumentSelection()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelection::SelectAll()
{
	ListView->Items->BeginUpdate();
	try {
		ListView->SelectAll();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelection::ReverseSelection()
{
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::ReverseSelection(ListView);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentCommands::EnsureValidPath(const AnsiString& APath)
{
	AnsiString LPath;
	try {
		TPathUtilities::AssureAbsolute(APath);
		LPath = TShortcutFileUtilities::EnsureDereferenced(APath);
		LPath = ExpandFileName(APath);
		LPath = TFileUtilities::GetLongPath(LPath);
	}
	catch(EFileError& LException) {
		throw EAppError(AnsiString().sprintf("The program cannot accept the path \"%s\".", APath.c_str()));
	}
	catch(EShellLinkError& LException) {
		throw EAppError(AnsiString().sprintf("The program cannot accept the path \"%s\".", APath.c_str()));
	}
	return LPath;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Initialize()
{
	try {
		FOpenInvoker = new TInvoker();
		FEditInvoker = new TInvoker();
		FClipboardWatcher = new TClipboardWatcher();
		FClipboardWatcher->Enabled = true;
		FClipboardWatcher->OnChange = ClipboardWatcherChange;
		FOpenDialog = new TOpenDialog(NULL);
		FOpenDialog->Title = "Open";
		FOpenDialog->Filter = "D77 disk images (*.d77)|*.d77|D88 disk images (*.d88)|*.d88|All disk images|*.d77;*.d88|All (*.*)|*.*";
		FOpenDialog->FilterIndex = 3;
		FOpenDialog->Options = FOpenDialog->Options
			<< ofFileMustExist;
		FSaveAsDialog = new TSaveDialog(NULL);
		FSaveAsDialog->Title = "Save As";
		FSaveAsDialog->Filter = "D77 disk images (*.d77)|*.d77|D88 disk images (*.d88)|*.d88";
		FSaveAsDialog->FilterIndex = 2;
		FSaveAsDialog->Options = FSaveAsDialog->Options
			<< ofOverwritePrompt
			<< ofPathMustExist;
		FSaveAsDialog->OnTypeChange = SaveAsDialogTypeChange;
		FSaveAsDialog->OnTypeChange(FSaveAsDialog);
		FAddExistentDialog = new TOpenDialog(NULL);
		FAddExistentDialog->Title = "Add Existent";
		FAddExistentDialog->Filter = "D77 disk images (*.d77)|*.d77|D88 disk images (*.d88)|*.d88|All disk images|*.d77;*.d88|All (*.*)|*.*";
		FAddExistentDialog->FilterIndex = 3;
		FAddExistentDialog->Options = FAddExistentDialog->Options
			<< ofAllowMultiSelect
			<< ofFileMustExist;
		FMergeDialog = new TSaveDialog(NULL);
		FMergeDialog->Title = "Merge";
		FMergeDialog->Filter = "D77 disk images (*.d77)|*.d77|D88 disk images (*.d88)|*.d88";
		FMergeDialog->FilterIndex = 2;
		FMergeDialog->Options = FMergeDialog->Options
			<< ofOverwritePrompt
			<< ofPathMustExist;
		FMergeDialog->OnTypeChange = MergeDialogTypeChange;
		FMergeDialog->OnTypeChange(FMergeDialog);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Uninitialize()
{
	delete FMergeDialog;
	FMergeDialog = NULL;

	delete FAddExistentDialog;
	FAddExistentDialog = NULL;

	delete FSaveAsDialog;
	FSaveAsDialog = NULL;

	delete FOpenDialog;
	FOpenDialog = NULL;

	delete FClipboardWatcher;
	FClipboardWatcher = NULL;

	delete FEditInvoker;
	FEditInvoker = NULL;

	delete FOpenInvoker;
	FOpenInvoker = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Open(const AnsiString& AFile)
{
	TAutoPointer<TDocumentCommand> LCommand;
	try {
		TAutoPointer<TObjectList> LList = new TObjectList();
		{
			TAutoPointer<TMemoryStream> LStream = new TMemoryStream();
			LStream->LoadFromFile(AFile);
			LStream->Position = 0;
			while(LStream->Position < LStream->Size) {
				TAutoPointer<TDiskImage> LItem = new TDiskImage();
				LItem->LoadFromStream(LStream);
				LList->Add(LItem);
				LItem.Release();
			}
		}
		DynamicArray<TDiskImage*> LItems;
		LItems.Length = LList->Count;
		for(int LI = 0; LI < LList->Count; ++LI) {
			LItems[LI] = static_cast<TDiskImage*>(LList->Items[LI]);
		}
		if(CanNew()) {
			New();
		}
		LCommand = new TAddCommand(DocumentListModel, LItems);
		LList->OwnsObjects = false;
		LCommand->Description = "Open";
	}
	catch(EStreamError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program could not open the document \"%s\".",
			AFile.c_str()));
	}
	OpenInvoker->Add(LCommand);
	LCommand.Release();
	OpenInvoker->Execute();
	DocumentFileModel->File = AFile;
	HistoryModel->Add(AFile);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SaveAs(const AnsiString& AFile)
{
	try {
		TAutoPointer<TMemoryStream> LStream = new TMemoryStream();
		for(int LI = 0; LI < DocumentListModel->Count; ++LI) {
			DocumentListModel->Items[LI]->SaveToStream(LStream);
		}
		LStream->SaveToFile(AFile);
		DocumentFileModel->File = AFile;
		HistoryModel->Add(AFile);
		EditInvoker->ResetOrigin();
	}
	catch(EStreamError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program could not save the document as \"%s\".",
			AFile.c_str()));
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddExistent(const TStringDynArray& AFiles)
{
	TAutoPointer<TDocumentCommand> LCommand;
	{
		TAutoPointer<TObjectList> LList = new TObjectList();
		for(int LI = 0; LI < AFiles.Length; ++LI) {
			try {
				TAutoPointer<TMemoryStream> LStream = new TMemoryStream();
				LStream->LoadFromFile(AFiles[LI]);
				LStream->Position = 0;
				while(LStream->Position < LStream->Size) {
					TAutoPointer<TDiskImage> LItem = new TDiskImage();
					LItem->LoadFromStream(LStream);
					LList->Add(LItem);
					LItem.Release();
				}
			}
			catch(EStreamError& LException) {
				throw EAppError(AnsiString().sprintf(
					"The program could not open the document \"%s\".",
					AFiles[LI].c_str()));
			}
		}
		DynamicArray<TDiskImage*> LItems;
		LItems.Length = LList->Count;
		for(int LI = 0; LI < LList->Count; ++LI) {
			LItems[LI] = static_cast<TDiskImage*>(LList->Items[LI]);
		}
		LCommand = new TAddCommand(DocumentListModel, LItems);
		LList->OwnsObjects = false;
		LCommand->Description = "Add existent";
	}
	EditInvoker->Add(LCommand);
	LCommand.Release();
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddBlank(int ADiskType)
{
	TAutoPointer<TAddCommand> LCommand;
	{
		{
			TAutoPointer<TDiskImage> LItem = new TDiskImage(ADiskType);
			LCommand = new TAddCommand(DocumentListModel, TDynamicArrayUtilities<TDiskImage*>::FromElements(LItem));
			LItem.Release();
		}
		switch(ADiskType) {
		case TDiskImageFormat::DiskType_2D:
			LCommand->Description = "Add blank 2D";
			break;
		case TDiskImageFormat::DiskType_2DD:
			LCommand->Description = "Add blank 2DD";
			break;
		case TDiskImageFormat::DiskType_2HD:
			LCommand->Description = "Add blank 2HD";
			break;
		default:
			throw EAppError("The program cannot accept the disk type.");
		}
	}
	EditInvoker->Add(LCommand);
	LCommand.Release();
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory(int AIndex)
{
	Open(HistoryModel->Items[AIndex]);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ClipboardWatcherChange(TObject* ASender)
{
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SaveAsDialogTypeChange(TObject* ASender)
{
	TSaveDialog* LSaveDialog = static_cast<TSaveDialog*>(ASender);
	switch(LSaveDialog->FilterIndex) {
	case 1:
		LSaveDialog->DefaultExt = "d77";
		break;
	case 2:
		LSaveDialog->DefaultExt = "d88";
		break;
	default:
		throw EAppError("The program cannot accept the file type.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::MergeDialogTypeChange(TObject* ASender)
{
	TSaveDialog* LSaveDialog = static_cast<TSaveDialog*>(ASender);
	switch(LSaveDialog->FilterIndex) {
	case 1:
		LSaveDialog->DefaultExt = "d77";
		break;
	case 2:
		LSaveDialog->DefaultExt = "d88";
		break;
	default:
		throw EAppError("The program cannot accept the file type.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentCommands::GetUndoDescription()
{
	if(CanUndo()) {
		return static_cast<TDocumentCommand*>(EditInvoker->NextUndo)->Description;
	}
	else {
		return "(None)";
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentCommands::GetRedoDescription()
{
	if(CanRedo()) {
		return static_cast<TDocumentCommand*>(EditInvoker->NextRedo)->Description;
	}
	else {
		return "(None)";
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommands::TDocumentCommands(TDocumentFileModel* ADocumentFileModel, TDocumentListModel* ADocumentListModel, TDocumentSelection* ADocumentSelection, THistoryModel* AHistoryModel) :
	TCommands(),
	FDocumentFileModel(ADocumentFileModel),
	FDocumentListModel(ADocumentListModel),
	FDocumentSelection(ADocumentSelection),
	FHistoryModel(AHistoryModel),
	FOpenInvoker(NULL),
	FEditInvoker(NULL),
	FClipboardWatcher(NULL),
	FOpenDialog(NULL),
	FSaveAsDialog(NULL),
	FAddExistentDialog(NULL),
	FMergeDialog(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommands::~TDocumentCommands()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::New()
{
	if(!CanNew()) {
		HandleFeatureUnavailable();
	}
	if(CanSave()) {
		switch(Application->MessageBox(
			"Do you want to save the current document?",
			Application->Title.c_str(),
			MB_YESNOCANCEL | MB_ICONEXCLAMATION | MB_SETFOREGROUND)) {
		case IDYES:
			Save();
			break;
		case IDNO:
			// Do nothing
			break;
		default:
			Abort();
		}
	}
	DocumentListModel->Clear();
	DocumentFileModel->File = AnsiString();
	OpenInvoker->ClearHistory();
	EditInvoker->ClearHistory();
	EditInvoker->ResetOrigin();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanNew()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Open()
{
	if(!CanOpen()) {
		HandleFeatureUnavailable();
	}
	OpenDialog->FileName = AnsiString();
	if(!OpenDialog->Execute()) {
		Abort();
	}
	Open(OpenDialog->FileName);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpen()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Save()
{
	if(!CanSave()) {
		HandleFeatureUnavailable();
	}
	if(!DocumentFileModel->IsOpened()) {
		SaveAs();
		return;
	}
	SaveAs(DocumentFileModel->File);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSave()
{
	return EditInvoker->IsModified();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SaveAs()
{
	if(!CanSaveAs()) {
		HandleFeatureUnavailable();
	}
	if(DocumentFileModel->IsOpened()) {
		SaveAsDialog->FileName = DocumentFileModel->File;
	}
	else {
		SaveAsDialog->FileName = DocumentFileModel->Title;
	}
	if(!SaveAsDialog->Execute()) {
		Abort();
	}
	SaveAs(SaveAsDialog->FileName);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSaveAs()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddExistent()
{
	if(!CanAddExistent()) {
		HandleFeatureUnavailable();
	}
	AddExistentDialog->FileName = AnsiString();
	if(!AddExistentDialog->Execute()) {
		Abort();
	}
	AddExistent(TAnsiStringDynamicArrayUtilities::FromList(AddExistentDialog->Files));
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanAddExistent()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddBlank2D()
{
	if(!CanAddBlank2D()) {
		HandleFeatureUnavailable();
	}
	AddBlank(TDiskImageFormat::DiskType_2D);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanAddBlank2D()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddBlank2DD()
{
	if(!CanAddBlank2DD()) {
		HandleFeatureUnavailable();
	}
	AddBlank(TDiskImageFormat::DiskType_2DD);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanAddBlank2DD()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddBlank2HD()
{
	if(!CanAddBlank2HD()) {
		HandleFeatureUnavailable();
	}
	AddBlank(TDiskImageFormat::DiskType_2HD);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanAddBlank2HD()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Merge()
{
	if(!CanMerge()) {
		HandleFeatureUnavailable();
	}
	MergeDialog->FileName = DocumentFileModel->Title;
	if(!MergeDialog->Execute()) {
		Abort();
	}
	try {
		TAutoPointer<TMemoryStream> LStream = new TMemoryStream();
		{
			TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				DocumentListModel->Items[LIndexes[LI]]->SaveToStream(LStream);
			}
		}
		LStream->SaveToFile(MergeDialog->FileName);
	}
	catch(EStreamError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program could not save the document as \"%s\".",
			MergeDialog->FileName.c_str()));
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanMerge()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Split()
{
	if(!CanSplit()) {
		HandleFeatureUnavailable();
	}
	TAutoPointer<TSplitPathProducer> LProducer;
	{
		TAppSplitDialog::EnsureCreate();
		TSplitArguments LValues = AppSplitDialog->Values;
		LValues.Directory = GetCurrentDir();
		LValues.Archive = ChangeFileExt(DocumentFileModel->Title, AnsiString());
		LValues.Extension = ExtractFileExt(DocumentFileModel->Title);
		LValues.Extension = TPathUtilities::EnsureStartWithoutCharacter(LValues.Extension, '.');
		if(LValues.Extension.Length() == 0) {
			LValues.Extension = "d88";
		}
		AppSplitDialog->Values = LValues;
		if(!AppSplitDialog->Execute()) {
			Abort();
		}
		LValues = AppSplitDialog->Values;
		LProducer = new TSplitPathProducer();
		LProducer->Directory = LValues.Directory;
		LProducer->Archive = LValues.Archive;
		LProducer->EntryPrefix = LValues.EntryPrefix;
		LProducer->EntrySuffix = LValues.EntrySuffix;
		LProducer->Extension = LValues.Extension;
	}
	ForceDirectories(LProducer->Directory);
	TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		TDiskImage* LItem = DocumentListModel->Items[LIndexes[LI]];
		LItem->SaveToFile(LProducer->Produce(LItem->Name));
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSplit()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory1()
{
	if(!CanOpenHistory1()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(0);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory1()
{
	return HistoryModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory2()
{
	if(!CanOpenHistory2()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(1);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory2()
{
	return HistoryModel->Count > 1;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory3()
{
	if(!CanOpenHistory3()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(2);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory3()
{
	return HistoryModel->Count > 2;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory4()
{
	if(!CanOpenHistory4()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(3);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory4()
{
	return HistoryModel->Count > 3;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory5()
{
	if(!CanOpenHistory5()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(4);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory5()
{
	return HistoryModel->Count > 4;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory6()
{
	if(!CanOpenHistory6()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(5);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory6()
{
	return HistoryModel->Count > 5;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory7()
{
	if(!CanOpenHistory7()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(6);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory7()
{
	return HistoryModel->Count > 6;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory8()
{
	if(!CanOpenHistory8()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(7);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory8()
{
	return HistoryModel->Count > 7;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::OpenHistory9()
{
	if(!CanOpenHistory9()) {
		HandleFeatureUnavailable();
	}
	OpenHistory(8);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanOpenHistory9()
{
	return HistoryModel->Count > 8;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ClearHistory()
{
	if(!CanClearHistory()) {
		HandleFeatureUnavailable();
	}
	if(Application->MessageBox(
		"Do you want to clear the history?",
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	HistoryModel->Clear();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanClearHistory()
{
	return HistoryModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Undo()
{
	if(!CanUndo()) {
		HandleFeatureUnavailable();
	}
	EditInvoker->Undo();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanUndo()
{
	return EditInvoker->CanUndo();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Redo()
{
	if(!CanRedo()) {
		HandleFeatureUnavailable();
	}
	EditInvoker->Redo();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanRedo()
{
	return EditInvoker->CanRedo();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Duplicate()
{
	if(!CanDuplicate()) {
		HandleFeatureUnavailable();
	}
	TAutoPointer<TDocumentCommand> LCommand;
	{
		TAutoPointer<TObjectList> LList = new TObjectList();
		{
			TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				TAutoPointer<TDiskImage> LItem = new TDiskImage();
				LItem->Assign(DocumentListModel->Items[LIndexes[LI]]);
				LList->Add(LItem);
				LItem.Release();
			}
		}
		DynamicArray<TDiskImage*> LItems;
		LItems.Length = LList->Count;
		for(int LI = 0; LI < LList->Count; ++LI) {
			LItems[LI] = static_cast<TDiskImage*>(LList->Items[LI]);
		}
		LCommand = new TAddCommand(DocumentListModel, LItems);
		LList->OwnsObjects = false;
		LCommand->Description = "Duplicate";
	}
	EditInvoker->Add(LCommand);
	LCommand.Release();
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanDuplicate()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Delete()
{
	if(!CanDelete()) {
		HandleFeatureUnavailable();
	}
	TAutoPointer<TDocumentCommand> LCommand = new TDeleteCommand(DocumentListModel, DocumentSelection->SelectionIndexes);
	LCommand->Description = "Delete";
	EditInvoker->Add(LCommand);
	LCommand.Release();
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanDelete()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SelectAll()
{
	if(!CanSelectAll()) {
		HandleFeatureUnavailable();
	}
	DocumentSelection->SelectAll();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSelectAll()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ReverseSelection()
{
	if(!CanReverseSelection()) {
		HandleFeatureUnavailable();
	}
	DocumentSelection->ReverseSelection();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanReverseSelection()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByName()
{
	if(!CanSortByName()) {
		HandleFeatureUnavailable();
	}
	DocumentListModel->SortByName();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByName()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByDiskType()
{
	if(!CanSortByDiskType()) {
		HandleFeatureUnavailable();
	}
	DocumentListModel->SortByDiskType();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByDiskType()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByWriteProtected()
{
	if(!CanSortByWriteProtected()) {
		HandleFeatureUnavailable();
	}
	DocumentListModel->SortByWriteProtected();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByWriteProtected()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortBySize()
{
	if(!CanSortBySize()) {
		HandleFeatureUnavailable();
	}
	DocumentListModel->SortBySize();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortBySize()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ReverseOrder()
{
	if(!CanReverseOrder()) {
		HandleFeatureUnavailable();
	}
	DocumentListModel->ReverseOrder();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanReverseOrder()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::MoveUp()
{
	if(!CanMoveUp()) {
		HandleFeatureUnavailable();
	}
	DocumentListModel->MoveUp(DocumentSelection->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanMoveUp()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::MoveDown()
{
	if(!CanMoveDown()) {
		HandleFeatureUnavailable();
	}
	DocumentListModel->MoveDown(DocumentSelection->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanMoveDown()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::CopyName()
{
	if(!CanCopyName()) {
		HandleFeatureUnavailable();
	}
	TAutoPointer<TStrings> LLines = new TStringList();
	{
		TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
		LLines->Capacity = LIndexes.Length;
		for(int LI = 0; LI < LIndexes.Length; ++LI) {
			LLines->Add(DocumentListModel->Items[LIndexes[LI]]->Name);
		}
	}
	Clipboard()->AsText = LLines->Text;
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanCopyName()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::PasteName()
{
	if(!CanPasteName()) {
		HandleFeatureUnavailable();
	}
	{
		TAutoPointer<TSetPropertyCommand> LCommand = new TSetPropertyCommand(DocumentListModel);
		{
			TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
			TAutoPointer<TStrings> LValues = new TStringList();
			LValues->Text = Clipboard()->AsText;
			if(LIndexes.Length != LValues->Count) {
				throw EAppError("The program cannot paste the text in the clipboard.");
			}
			for(int LI = 0; LI < LValues->Count; ++LI) {
				LValues->Strings[LI] = TDiskImageFormat::AssureValidName(LValues->Strings[LI]);
			}
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				TAutoPointer<TSetNameSubCommand> LSubCommand = new TSetNameSubCommand(
					DocumentListModel,
					DocumentListModel->Items[LIndexes[LI]],
					LValues->Strings[LI]);
				LCommand->Invoker->Add(LSubCommand);
				LSubCommand.Release();
			}
		}
		LCommand->Description = "Paste name";
		EditInvoker->Add(LCommand);
		LCommand.Release();
	}
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanPasteName()
{
	return DocumentSelection->SelectionCount > 0 && Clipboard()->HasFormat(CF_TEXT);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::TypeName()
{
	if(!CanTypeName()) {
		HandleFeatureUnavailable();
	}
	{
		TAutoPointer<TSetPropertyCommand> LCommand = new TSetPropertyCommand(DocumentListModel);
		{
			TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
			AnsiString LValue = DocumentListModel->Items[DocumentSelection->SelectionIndexes[0]]->Name;
			if(!TAppInputDialog::EnsureCreate()->Execute(
				"Type Name",
				AnsiString().sprintf("Type the name (up to %d characters in ASCII).", TDiskImageFormat::MaxNameLength),
				LValue)) {
				Abort();
			}
			TDiskImageFormat::AssureValidName(LValue);
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				TAutoPointer<TSetNameSubCommand> LSubCommand = new TSetNameSubCommand(
					DocumentListModel,
					DocumentListModel->Items[LIndexes[LI]],
					LValue);
				LCommand->Invoker->Add(LSubCommand);
				LSubCommand.Release();
			}
		}
		LCommand->Description = "Type name";
		EditInvoker->Add(LCommand);
		LCommand.Release();
	}
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanTypeName()
{
	return DocumentSelection->SelectionCount == 1;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::TurnOnWriteProtection()
{
	if(!CanTurnOnWriteProtection()) {
		HandleFeatureUnavailable();
	}
	{
		TAutoPointer<TSetPropertyCommand> LCommand = new TSetPropertyCommand(DocumentListModel);
		{
			TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				TAutoPointer<TSetWriteProtectedSubCommand> LSubCommand = new TSetWriteProtectedSubCommand(
					DocumentListModel,
					DocumentListModel->Items[LIndexes[LI]],
					TDiskImageFormat::WriteProtected_Yes);
				LCommand->Invoker->Add(LSubCommand);
				LSubCommand.Release();
			}
		}
		LCommand->Description = "Turn on write-protection";
		EditInvoker->Add(LCommand);
		LCommand.Release();
	}
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanTurnOnWriteProtection()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::TurnOffWriteProtection()
{
	if(!CanTurnOffWriteProtection()) {
		HandleFeatureUnavailable();
	}
	{
		TAutoPointer<TSetPropertyCommand> LCommand = new TSetPropertyCommand(DocumentListModel);
		{
			TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				TAutoPointer<TSetWriteProtectedSubCommand> LSubCommand = new TSetWriteProtectedSubCommand(
					DocumentListModel,
					DocumentListModel->Items[LIndexes[LI]],
					TDiskImageFormat::WriteProtected_No);
				LCommand->Invoker->Add(LSubCommand);
				LSubCommand.Release();
			}
		}
		LCommand->Description = "Turn off write-protection";
		EditInvoker->Add(LCommand);
		LCommand.Release();
	}
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanTurnOffWriteProtection()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ToggleWriteProtection()
{
	if(!CanToggleWriteProtection()) {
		HandleFeatureUnavailable();
	}
	{
		TAutoPointer<TSetPropertyCommand> LCommand = new TSetPropertyCommand(DocumentListModel);
		{
			TIntegerDynArray LIndexes = DocumentSelection->SelectionIndexes;
			for(int LI = 0; LI < LIndexes.Length; ++LI) {
				TAutoPointer<TSetWriteProtectedSubCommand> LSubCommand = new TSetWriteProtectedSubCommand(
					DocumentListModel,
					DocumentListModel->Items[LIndexes[LI]],
					TDiskImageFormat::ToggleWriteProtected(DocumentListModel->Items[LIndexes[LI]]->WriteProtected));
				LCommand->Invoker->Add(LSubCommand);
				LSubCommand.Release();
			}
		}
		LCommand->Description = "Toggle write-protection";
		EditInvoker->Add(LCommand);
		LCommand.Release();
	}
	EditInvoker->Execute();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanToggleWriteProtection()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ShowTotalSize()
{
	if(!CanShowTotalSize()) {
		HandleFeatureUnavailable();
	}
	__int64 LSize = 0;
	{
		TIntegerDynArray LSelectionIndexes = DocumentSelection->SelectionIndexes;
		for(int LI = 0; LI < LSelectionIndexes.Length; ++LI) {
			LSize += DocumentListModel->Items[LSelectionIndexes[LI]]->Size;
		}
	}
	Application->MessageBox(
		AnsiString().sprintf("The total size is %s bytes.", TCommon::FormatInteger(LSize).c_str()).c_str(),
		"Total Size",
		MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanShowTotalSize()
{
	return DocumentSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::DropFiles(const TStringDynArray& AFiles)
{
	if(!CanDropFiles()) {
		HandleFeatureUnavailable();
	}
	if(AFiles.Length == 0) {
		return;
	}
	TStringDynArray LFiles = AFiles;
	for(int LI = 0; LI < LFiles.Length; ++LI) {
		LFiles[LI] = EnsureValidPath(LFiles[LI]);
	}
	if(LFiles.Length == 1 && !DocumentFileModel->IsOpened() && !CanUndo() && !CanRedo()) {
		Open(LFiles[0]);
	}
	else {
		AddExistent(LFiles);
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanDropFiles()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".OpenDialog";
	{
		OpenDialog->FilterIndex = ASettings->ReadInteger(
			LSection,
			"FilterIndex",
			OpenDialog->FilterIndex);
	}
	LSection = ASection + ".SaveAsDialog";
	{
		SaveAsDialog->FilterIndex = ASettings->ReadInteger(
			LSection,
			"FilterIndex",
			SaveAsDialog->FilterIndex);
		if(SaveAsDialog->OnTypeChange != NULL) {
			SaveAsDialog->OnTypeChange(SaveAsDialog);
		}
	}
	LSection = ASection + ".AddExistentDialog";
	{
		AddExistentDialog->FilterIndex = ASettings->ReadInteger(
			LSection,
			"FilterIndex",
			AddExistentDialog->FilterIndex);
	}
	LSection = ASection + ".MergeDialog";
	{
		MergeDialog->FilterIndex = ASettings->ReadInteger(
			LSection,
			"FilterIndex",
			MergeDialog->FilterIndex);
		if(MergeDialog->OnTypeChange != NULL) {
			MergeDialog->OnTypeChange(MergeDialog);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".OpenDialog";
	{
		ASettings->WriteInteger(
			LSection,
			"FilterIndex",
			OpenDialog->FilterIndex);
	}
	LSection = ASection + ".SaveAsDialog";
	{
		ASettings->WriteInteger(
			LSection,
			"FilterIndex",
			SaveAsDialog->FilterIndex);
	}
	LSection = ASection + ".AddExistentDialog";
	{
		ASettings->WriteInteger(
			LSection,
			"FilterIndex",
			AddExistentDialog->FilterIndex);
	}
	LSection = ASection + ".MergeDialog";
	{
		ASettings->WriteInteger(
			LSection,
			"FilterIndex",
			MergeDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
} // End namespace App_documents
//---------------------------------------------------------------------------

