//---------------------------------------------------------------------------
#ifndef App_DocumentsH
#define App_DocumentsH
//---------------------------------------------------------------------------
#include "App_DiskImages.h"
#include "Moa_Lists.h"
namespace App_history {class THistoryModel;} using namespace App_history;
namespace Comctrls {class TListView;} using namespace Comctrls;
namespace Dialogs {class TOpenDialog;} using namespace Dialogs;
namespace Dialogs {class TSaveDialog;} using namespace Dialogs;
namespace Forms {class TForm;} using namespace Forms;
namespace Moa_clipboard {class TClipboardWatcher;} using namespace Moa_clipboard;
namespace Moa_commandpattern {class TInvoker;} using namespace Moa_commandpattern;
//---------------------------------------------------------------------------
namespace App_documents {
//---------------------------------------------------------------------------
class TDocumentFileModel : public TModel
{
private:
	AnsiString FFile;
	TNotifyEvent FOnChange;
	void __fastcall SetFile(const AnsiString& AFile);
	AnsiString __fastcall GetTitle();
protected:
	virtual void __fastcall DoChange();
public:
	__fastcall TDocumentFileModel();
	virtual __fastcall ~TDocumentFileModel();
	virtual bool __fastcall IsOpened();
	__property AnsiString File = {read=FFile, write=SetFile};
	__property AnsiString Title = {read=GetTitle};
	__property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
};
//---------------------------------------------------------------------------
class TDocumentFileView : public TView
{
private:
	TDocumentFileModel* FDocumentFileModel;
	TForm* FForm;
	__property TDocumentFileModel* DocumentFileModel = {read=FDocumentFileModel};
	__property TForm* Form = {read=FForm};
protected:
public:
	__fastcall TDocumentFileView(TDocumentFileModel* ADocumentFileModel, TForm* AForm);
	virtual __fastcall ~TDocumentFileView();
	virtual void __fastcall HandleChange(TObject* ASender);
};
//---------------------------------------------------------------------------
class TDocumentListModel : public TModel
{
private:
	TList* FList;
	TListIndexesEvent FOnInsertItems;
	TListIndexesEvent FOnDeleteItems;
	TNotifyEvent FOnClear;
	TListReorderEvent FOnReorder;
	TNotifyEvent FOnFileChange;
	TListIndexEvent FOnItemChange;
	static int __fastcall SortCompareByName(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByDiskType(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByWriteProtected(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareBySize(int AIndex1, int AIndex2, void* AData);
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TDiskImage* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	__property TList* List = {read=FList};
protected:
	virtual void __fastcall DoInsertItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall DoDeleteItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall DoClear();
	virtual void __fastcall DoReorder(const TListReorderRecords& ARecords);
	virtual void __fastcall DoItemChange(int AIndex);
public:
	__fastcall TDocumentListModel();
	virtual __fastcall ~TDocumentListModel();
	virtual void __fastcall AddItems(const DynamicArray<TDiskImage*>& AItems);
	virtual void __fastcall DeleteItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall Clear();
	virtual void __fastcall SortByName();
	virtual void __fastcall SortByDiskType();
	virtual void __fastcall SortByWriteProtected();
	virtual void __fastcall SortBySize();
	virtual void __fastcall ReverseOrder();
	virtual void __fastcall MoveUp(const TIntegerDynArray& AIndexes);
	virtual void __fastcall MoveDown(const TIntegerDynArray& AIndexes);
	virtual void __fastcall Reorder(const TListReorderRecords& ARecords);
	virtual void __fastcall ItemChange(int AIndex);
	virtual int __fastcall Find(TDiskImage* ADiskImage);
	virtual TIntegerDynArray __fastcall FindItems(const DynamicArray<TDiskImage*>& AItems);
	virtual DynamicArray<TDiskImage*> __fastcall GetItemsAt(const TIntegerDynArray& AIndexes);
	__property TDiskImage* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
	__property TListIndexesEvent OnInsertItems = {read=FOnInsertItems, write=FOnInsertItems};
	__property TListIndexesEvent OnDeleteItems = {read=FOnDeleteItems, write=FOnDeleteItems};
	__property TNotifyEvent OnClear = {read=FOnClear, write=FOnClear};
	__property TListReorderEvent OnReorder = {read=FOnReorder, write=FOnReorder};
	__property TListIndexEvent OnItemChange = {read=FOnItemChange, write=FOnItemChange};
};
//---------------------------------------------------------------------------
class TDocumentListView : public TView
{
private:
	TDocumentListModel* FDocumentListModel;
	TListView* FListView;
	__property TDocumentListModel* DocumentListModel = {read=FDocumentListModel};
	__property TListView* ListView = {read=FListView};
protected:
public:
	__fastcall TDocumentListView(TDocumentListModel* ADocumentListModel, TListView* AListView);
	virtual __fastcall ~TDocumentListView();
	virtual void __fastcall HandleInsertItems(TObject* ASender, const TIntegerDynArray& AIndexes);
	virtual void __fastcall HandleDeleteItems(TObject* ASender, const TIntegerDynArray& AIndexes);
	virtual void __fastcall HandleClear(TObject* ASender);
	virtual void __fastcall HandleReorder(TObject* ASender, const TListReorderRecords& ARecords);
	virtual void __fastcall HandleItemChange(TObject* ASender, int AIndex);
};
//---------------------------------------------------------------------------
class TDocumentSelection : public TModel
{
private:
	TListView* FListView;
	bool __fastcall GetItem(int AIndex);
	void __fastcall SetItem(int AIndex, bool AItem);
	int __fastcall GetCount();
	int __fastcall GetSelectionCount();
	TIntegerDynArray __fastcall GetSelectionIndexes();
	void __fastcall SetSelectionIndexes(const TIntegerDynArray& ASelectionIndexes);
	__property TListView* ListView = {read=FListView};
protected:
public:
	__fastcall TDocumentSelection(TListView* AListView);
	virtual __fastcall ~TDocumentSelection();
	virtual void __fastcall SelectAll();
	virtual void __fastcall ReverseSelection();
	__property bool Items[int AIndex] = {read=GetItem, write=SetItem};
	__property int Count = {read=GetCount};
	__property int SelectionCount = {read=GetSelectionCount};
	__property TIntegerDynArray SelectionIndexes = {read=GetSelectionIndexes, write=SetSelectionIndexes};
};
//---------------------------------------------------------------------------
class TDocumentCommands : public TCommands
{
private:
	TDocumentFileModel* FDocumentFileModel;
	TDocumentListModel* FDocumentListModel;
	TDocumentSelection* FDocumentSelection;
	THistoryModel* FHistoryModel;
	TInvoker* FOpenInvoker;
	TInvoker* FEditInvoker;
	TClipboardWatcher* FClipboardWatcher;
	TOpenDialog* FOpenDialog;
	TSaveDialog* FSaveAsDialog;
	TOpenDialog* FAddExistentDialog;
	TSaveDialog* FMergeDialog;
	TNotifyEvent FOnChange;
	static AnsiString __fastcall EnsureValidPath(const AnsiString& APath);
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall Open(const AnsiString& AFile);
	void __fastcall SaveAs(const AnsiString& AFile);
	void __fastcall AddExistent(const TStringDynArray& AFiles);
	void __fastcall AddBlank(int ADiskType);
	void __fastcall OpenHistory(int AIndex);
	void __fastcall ClipboardWatcherChange(TObject* ASender);
	void __fastcall SaveAsDialogTypeChange(TObject* ASender);
	void __fastcall MergeDialogTypeChange(TObject* ASender);
	AnsiString __fastcall GetUndoDescription();
	AnsiString __fastcall GetRedoDescription();
	__property TDocumentFileModel* DocumentFileModel = {read=FDocumentFileModel};
	__property TDocumentListModel* DocumentListModel = {read=FDocumentListModel};
	__property TDocumentSelection* DocumentSelection = {read=FDocumentSelection};
	__property THistoryModel* HistoryModel = {read=FHistoryModel};
	__property TInvoker* OpenInvoker = {read=FOpenInvoker};
	__property TInvoker* EditInvoker = {read=FEditInvoker};
	__property TClipboardWatcher* ClipboardWatcher = {read=FClipboardWatcher};
	__property TOpenDialog* OpenDialog = {read=FOpenDialog};
	__property TSaveDialog* SaveAsDialog = {read=FSaveAsDialog};
	__property TOpenDialog* AddExistentDialog = {read=FAddExistentDialog};
	__property TSaveDialog* MergeDialog = {read=FMergeDialog};
protected:
	virtual void __fastcall DoChange();
public:
	__fastcall TDocumentCommands(TDocumentFileModel* ADocumentFileModel, TDocumentListModel* ADocumentListModel, TDocumentSelection* ADocumentSelection, THistoryModel* AHistoryModel);
	virtual __fastcall ~TDocumentCommands();
	virtual void __fastcall New();
	virtual bool __fastcall CanNew();
	virtual void __fastcall Open();
	virtual bool __fastcall CanOpen();
	virtual void __fastcall Save();
	virtual bool __fastcall CanSave();
	virtual void __fastcall SaveAs();
	virtual bool __fastcall CanSaveAs();
	virtual void __fastcall AddExistent();
	virtual bool __fastcall CanAddExistent();
	virtual void __fastcall AddBlank2D();
	virtual bool __fastcall CanAddBlank2D();
	virtual void __fastcall AddBlank2DD();
	virtual bool __fastcall CanAddBlank2DD();
	virtual void __fastcall AddBlank2HD();
	virtual bool __fastcall CanAddBlank2HD();
	virtual void __fastcall Merge();
	virtual bool __fastcall CanMerge();
	virtual void __fastcall Split();
	virtual bool __fastcall CanSplit();
	virtual void __fastcall OpenHistory1();
	virtual bool __fastcall CanOpenHistory1();
	virtual void __fastcall OpenHistory2();
	virtual bool __fastcall CanOpenHistory2();
	virtual void __fastcall OpenHistory3();
	virtual bool __fastcall CanOpenHistory3();
	virtual void __fastcall OpenHistory4();
	virtual bool __fastcall CanOpenHistory4();
	virtual void __fastcall OpenHistory5();
	virtual bool __fastcall CanOpenHistory5();
	virtual void __fastcall OpenHistory6();
	virtual bool __fastcall CanOpenHistory6();
	virtual void __fastcall OpenHistory7();
	virtual bool __fastcall CanOpenHistory7();
	virtual void __fastcall OpenHistory8();
	virtual bool __fastcall CanOpenHistory8();
	virtual void __fastcall OpenHistory9();
	virtual bool __fastcall CanOpenHistory9();
	virtual void __fastcall ClearHistory();
	virtual bool __fastcall CanClearHistory();
	virtual void __fastcall Undo();
	virtual bool __fastcall CanUndo();
	virtual void __fastcall Redo();
	virtual bool __fastcall CanRedo();
	virtual void __fastcall Delete();
	virtual void __fastcall Duplicate();
	virtual bool __fastcall CanDuplicate();
	virtual bool __fastcall CanDelete();
	virtual void __fastcall SelectAll();
	virtual bool __fastcall CanSelectAll();
	virtual void __fastcall ReverseSelection();
	virtual bool __fastcall CanReverseSelection();
	virtual void __fastcall SortByName();
	virtual bool __fastcall CanSortByName();
	virtual void __fastcall SortByDiskType();
	virtual bool __fastcall CanSortByDiskType();
	virtual void __fastcall SortByWriteProtected();
	virtual bool __fastcall CanSortByWriteProtected();
	virtual void __fastcall SortBySize();
	virtual bool __fastcall CanSortBySize();
	virtual void __fastcall ReverseOrder();
	virtual bool __fastcall CanReverseOrder();
	virtual void __fastcall MoveUp();
	virtual bool __fastcall CanMoveUp();
	virtual void __fastcall MoveDown();
	virtual bool __fastcall CanMoveDown();
	virtual void __fastcall CopyName();
	virtual bool __fastcall CanCopyName();
	virtual void __fastcall PasteName();
	virtual bool __fastcall CanPasteName();
	virtual void __fastcall TypeName();
	virtual bool __fastcall CanTypeName();
	virtual void __fastcall TurnOnWriteProtection();
	virtual bool __fastcall CanTurnOnWriteProtection();
	virtual void __fastcall TurnOffWriteProtection();
	virtual bool __fastcall CanTurnOffWriteProtection();
	virtual void __fastcall ToggleWriteProtection();
	virtual bool __fastcall CanToggleWriteProtection();
	virtual void __fastcall ShowTotalSize();
	virtual bool __fastcall CanShowTotalSize();
	virtual void __fastcall DropFiles(const TStringDynArray& AFiles);
	virtual bool __fastcall CanDropFiles();
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	__property AnsiString UndoDescription = {read=GetUndoDescription};
	__property AnsiString RedoDescription = {read=GetRedoDescription};
	__property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
};
//---------------------------------------------------------------------------
} // End namespace App_documents
using namespace App_documents;
//---------------------------------------------------------------------------
#endif

