//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <ActnList.hpp>
#include <IniFiles.hpp>
#include "App_History.h"
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_history {
//---------------------------------------------------------------------------
void __fastcall THistoryModel::Initialize()
{
	try {
		FList = new TPathList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall THistoryModel::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
AnsiString __fastcall THistoryModel::GetItem(int AIndex)
{
	return List->Strings[AIndex];
}
//---------------------------------------------------------------------------
int __fastcall THistoryModel::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
void __fastcall THistoryModel::SetLimit(int ALimit)
{
	if(ALimit < 0) {
		throw EAppError("The program cannot set the limit.");
	}
	if(ALimit == Limit) {
		return;
	}
	while(Count > ALimit) {
		List->Delete(List->Count - 1);
	}
	FLimit = ALimit;
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall THistoryModel::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall THistoryModel::THistoryModel() :
	TModel(),
	FList(NULL),
	FLimit(0),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall THistoryModel::~THistoryModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
int __fastcall THistoryModel::Add(const AnsiString& AItem)
{
	if(Limit == 0) {
		return -1;
	}
	{
		int LIndex = Find(AItem);
		if(LIndex >= 0) {
			List->Delete(LIndex);
		}
	}
	if(List->Count == Limit) {
		List->Delete(List->Count - 1);
	}
	List->Insert(0, AItem);
	DoChange();
	return 0;
}
//---------------------------------------------------------------------------
int __fastcall THistoryModel::Remove(const AnsiString& AItem)
{
	int LIndex = Find(AItem);
	if(LIndex < 0) {
		return -1;
	}
	Delete(LIndex);
	return LIndex;
}
//---------------------------------------------------------------------------
void __fastcall THistoryModel::Delete(int AIndex)
{
	List->Delete(AIndex);
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall THistoryModel::Clear()
{
	List->Clear();
	DoChange();
}
//---------------------------------------------------------------------------
int __fastcall THistoryModel::Find(const AnsiString& AItem)
{
	return List->IndexOf(AItem);
}
//---------------------------------------------------------------------------
void __fastcall THistoryModel::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		Limit = ASettings->ReadInteger(
			LSection,
			"Limit",
			Limit);
		TAutoPointer<TStrings> LList = new TPathList();
		int LCount = ASettings->ReadInteger(
			LSection,
			"Count",
			0);
		LList->Capacity = LCount;
		for(int LI = 0; LI < LCount; ++LI) {
			AnsiString LItem = TAnsiStringUtilities::Dequote(ASettings->ReadString(
				LSection,
				AnsiString().sprintf("Item%d", LI),
				TAnsiStringUtilities::Quote(AnsiString())));
			LList->Add(LItem);
		}
		List->Assign(LList);
		DoChange();
	}
}
//---------------------------------------------------------------------------
void __fastcall THistoryModel::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		ASettings->WriteInteger(
			LSection,
			"Limit",
			Limit);
		ASettings->WriteInteger(
			LSection,
			"Count",
			Count);
		for(int LI = 0; LI < Count; ++LI) {
			ASettings->WriteString(
				LSection,
				AnsiString().sprintf("Item%d", LI),
				TAnsiStringUtilities::Quote(Items[LI]));
		}
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall THistoryView::Initialize()
{
	try {
		FList = new TList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall THistoryView::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TAction* __fastcall THistoryView::GetItem(int AIndex)
{
	return static_cast<TAction*>(List->Items[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall THistoryView::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
__fastcall THistoryView::THistoryView(THistoryModel* AHistoryModel) :
	TView(),
	FHistoryModel(AHistoryModel),
	FList(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall THistoryView::~THistoryView()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
int __fastcall THistoryView::Add(TAction* AItem)
{
	if(Find(AItem) >= 0) {
		throw EAppError("The program cannot add the item.");
	}
	return List->Add(AItem);
}
//---------------------------------------------------------------------------
int __fastcall THistoryView::Remove(TAction* AItem)
{
	int LIndex = Find(AItem);
	if(LIndex < 0) {
		return -1;
	}
	Delete(LIndex);
	return LIndex;
}
//---------------------------------------------------------------------------
void __fastcall THistoryView::Delete(int AIndex)
{
	List->Delete(AIndex);
}
//---------------------------------------------------------------------------
void __fastcall THistoryView::Clear()
{
	List->Clear();
}
//---------------------------------------------------------------------------
int __fastcall THistoryView::Find(TAction* AItem)
{
	return List->IndexOf(AItem);
}
//---------------------------------------------------------------------------
void __fastcall THistoryView::HandleChange(TObject* ASender)
{
	for(int LI = 0; LI < HistoryModel->Limit; ++LI) {
		AnsiString LItem;
		if(LI < HistoryModel->Count) {
			LItem = HistoryModel->Items[LI];
		}
		else {
			LItem = "(None)";
		}
		Items[LI]->Caption = AnsiString().sprintf(
			"&%d %s",
			LI + 1,
			TAnsiStringUtilities::DoubleEscape(LItem, '&').c_str());
	}
}
//---------------------------------------------------------------------------
} // End namespace App_history
//---------------------------------------------------------------------------

